﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.IO;
using System.Security.Permissions;

public class DirWatchDaemon
{

    // Incoming data from the client.
    public static string data = null;
    private static Socket handler = null;

    [PermissionSet(SecurityAction.Demand, Name = "FullTrust")]
    private static void StartListening(String directory)
    {
        // Data buffer for incoming data.
        byte[] bytes = new Byte[1024];

        // Establish the local endpoint for the socket.
        IPEndPoint localEndPoint = new IPEndPoint(System.Net.IPAddress.Parse("127.0.0.1"), 80);

        // Create a TCP/IP socket.
        Socket listener = new Socket(AddressFamily.InterNetwork,
            SocketType.Stream, ProtocolType.Tcp);

        Console.WriteLine("Waiting for a connection...");
        try
        {

            // Bind the socket to the local endpoint and 
            // listen for incoming connections.
            listener.Bind(localEndPoint);
            listener.Listen(10);
            handler = listener.Accept();

            try
            {
                // Start listening for connections.
                while (true)
                {
                    // Program is suspended while waiting for an incoming connection.
                    handler.Receive(new byte[1024]);
                    Console.WriteLine("CONNECTED");

                    // Create a new FileSystemWatcher and set its properties.
                    FileSystemWatcher watcher = new FileSystemWatcher();
                    watcher.Path = directory;

                    /* Watch for changes in LastAccess and LastWrite times, and
                       the renaming of files or directories. */
                    watcher.NotifyFilter = NotifyFilters.LastAccess | NotifyFilters.LastWrite
                       | NotifyFilters.FileName | NotifyFilters.DirectoryName;
                    // Only watch text files.
                    watcher.Filter = "*.*";

                    // Add event handlers.
                    watcher.Changed += new FileSystemEventHandler(OnChanged);
                    watcher.Created += new FileSystemEventHandler(OnChanged);
                    watcher.Deleted += new FileSystemEventHandler(OnChanged);
                    watcher.Renamed += new RenamedEventHandler(OnRenamed);

                    // Begin watching.
                    watcher.EnableRaisingEvents = true;

                }

            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }
            finally
            {
                handler.Shutdown(SocketShutdown.Both);
                handler.Close();
            }
        }
        catch (Exception e)
        {
            Console.WriteLine(e.ToString());
        }

        Console.WriteLine("\nPress ENTER to continue...");
        Console.Read();

    }
    
    // Define the event handlers. 
    private static void OnChanged(object source, FileSystemEventArgs e)
    {
        // Specify what is done when a file is changed, created, or deleted.
        writeToSocket(e.ChangeType + " " + e.FullPath);
    }

    private static void OnRenamed(object source, RenamedEventArgs e)
    {
        writeToSocket("Renamed " + e.OldFullPath + " -> " + e.FullPath);
    }

    private static void writeToSocket(String s)
    {
        byte[] msg = Encoding.ASCII.GetBytes(s + " " + DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"));
        handler.Send(msg);
    }

    public static int Main(String[] args)
    {

        // If a directory is not specified, exit program. 
        if (args.Length < 1)
        {
            // Display the proper way to call the program.
            Console.WriteLine("Usage: DirWatchDaemon.exe (directory)");
            return 1;
        }
        StartListening(args[0]);
        return 0;
    }
}